<?php
// For debugging/testing

if (!class_exists("PageSpeedBase")) {

  require_once 'EzOptions.php';
  require_once ABSPATH . 'wp-admin/includes/file.php';

  class PageSpeedBase extends EzBasePlugin {

    static $siteUrl;
    static $_options; // to use options in static functions
    static $cdnExt = array('jpg', 'jpeg', 'gif', 'png', 'js', 'css');
    static $qsRegEx = '/(jpg|jpeg|gif|png|js|css)\?.*?([\'"])/';
    static $cdnServer;

    function __construct($file) { //constructor
      parent::__construct("easy-pagespeed", "Easy PageSpeed", $file);
      $this->prefix = 'ezPageSpeed';
      $defaultOptions = $this->mkDefaultOptions();
      $this->optionName = $this->prefix;
      $this->options = get_option($this->optionName);
      if (empty($this->options)) {
        $this->options = $defaultOptions;
      }
      else {
        $this->options = array_merge($defaultOptions, $this->options);
      }
      if (!empty($this->options['pause_cdn'])) {
        return;
      }
      self::$cdnServer = trailingslashit(trim($this->options['cdn_server']));
      self::$siteUrl = trailingslashit(site_url());

      // Hooks and such
      if (!is_admin()) {
        add_action('parse_request', array($this, 'parseRequest'), 2);
      }
    }

    /**
     * Creates EzOption objects for admin page rendering
     */
    function mkEzOptions() {
      if (!empty($this->ezOptions)) {
        return;
      }
      $o = new EzCheckBox('pause_cdn');
      $o->title = __('Pause Easy PageSpeed temporarily. Your options will not be deleted, but the plugin will not do anything. Setting this option on is almost the same as deactivating the plugin.', 'easy-pagespeed');
      $o->desc = "<strong>" . __('Pause Easy PageSpeed?', 'easy-pagespeed') . "</strong>";
      $o->between = "&nbsp;";
      $o->after = "<br><br>";
      $o->tipWidth = "300";
      $this->ezOptions['pause_cdn'] = clone $o;

      $o = new EzCheckBox('kill_qs');
      $o->title = __('WordPress adds query strings to static resources indicating the version. They can cause issues with client browser caching. Turn on this option to remove query strings.', 'easy-pagespeed');
      $o->desc = __('Remove Query Strings?', 'easy-pagespeed');
      $o->between = "&nbsp;";
      $o->tipWidth = "300";
      $o->after = "<br><br>";
      $this->ezOptions['kill_qs'] = clone $o;

      $o = new EzCheckBox('show_timing');
      $o->title = __('This plugin can add a comment at the end of the HTML source code of each of your pages showing how long it took to do its processing. It is useful only for diagnostic purposes, and may interfere with some plugins such as Jetpack - Show Related Posts. Leave it unchecked unless you need to profile the plugin performance.', 'easy-pagespeed');
      $o->desc = __('Show Time Taken?', 'easy-pagespeed');
      $o->between = "&nbsp;";
      $o->tipWidth = "300";
      $o->after = "<br><br>";
      $this->ezOptions['show_timing'] = clone $o;

      $o = new EzCheckBox('enable_cdn');
      $o->title = __('Using a CDN can improve your page-loading performance. Use this option to indicate that you would like to serve static resources (files with extensions <code>jpg</code>, <code>jpeg</code>, <code>gif</code>, <code>png</code>, <code>js</code> and <code>css</code>) from the CDN url specified below.<br>For detailed info on CDNs,  see the yellow <strong>Inbo Box</strong>.', 'easy-pagespeed');
      $o->desc = __('Enable CDN?', 'easy-pagespeed');
      $o->between = "&nbsp;";
      $o->tipWidth = "300";
      $o->after = "<br><br>";
      $this->ezOptions['enable_cdn'] = clone $o;

      $o = new EzText('cdn_server');
      $o->title = __('Enter your CDN server URL (e.g. http://cdn.example.com) you would like to use for your static resources.<br>For detailed info on CDNs, see the yellow <strong>Inbo Box</strong>.', 'easy-pagespeed');
      $o->desc = __('CDN Server: &nbsp; &nbsp; ', 'easy-pagespeed');
      $o->style = "width:60%;";
      $o->tipWidth = "300";
      $o->after = "<br><br>";
      $this->ezOptions['cdn_server'] = clone $o;

      parent::mkEzOptions();
      $this->nLiteOptions = count($this->ezOptions);
    }

    /**
     * Makes the default values of options (for frontend as well as admin)
     * @return [string]
     */
    function mkDefaultOptions() {
      $defaultOptions = array(
          'pause_cdn' => false,
          'kill_qs' => true,
          'show_timing' => false,
          'enable_cdn' => true,
          'cdn_server' => '') +
              parent::mkDefaultOptions();
      return $defaultOptions;
    }

    /**
     * Handles submits from admin page
     * @return bool Returns empty if $_POST is empty
     */
    function handleSubmits() {
      if ($_SERVER["REQUEST_METHOD"] !== "POST") {
        return;
      }
      parent::handleSubmits();
      echo $this->adminMsg;
    }

    /**
     * Prints out divs that are shown in a neat box on the admin page.
     */
    function printInfoDivs() {
      require_once 'info-divs.php';
    }

    /**
     * Prints out the links that will display info divs
     */
    function printInfoLinks() {
      ?>
      <div style="background-color:#ffd;padding:10px;border:solid 1px #cca">
        <p style='margin-top:-8px;margin-bottom:-6px;float:left;text-align:center;font-size:1.1em'>
          <span style='background-color:#ffd'>
            &nbsp;<strong> Info Box</strong>&nbsp;
          </span>
        </p>
        <ul style="padding-left:30px;list-style-type:circle; list-style-position:inside;">
          <li><a href='#' class="cdn">What is a <strong>Content Delivery Network or CDN</strong>?</a></li>
          <li><a href='#' class="what">What does <strong>Easy PageSpeed</strong> do?</a></li>
          <li><a href='#' class="why">Why not let my caching plugin do this job?</a></li>
          <li><a href='#' class="pro-version">Why get the Pro version?</a></li>
        </ul>
      </div>
      <?php
    }

    /**
     * Prints out the admin page
     */
    function printAdminPage() {
      $ez = parent::printAdminPage();
      if (empty($ez)) {
        return;
      }
      $this->handleSubmits();
      $this->mkEzOptions();
      $this->setOptionValues();
      ?>
      <div class="wrap" style="width:1000px;">
        <?php
        echo <<<EOF1
    <h2>{$this->name}{$this->strPro}<a href="http://validator.w3.org/" target="_blank"><img src="http://www.w3.org/Icons/valid-xhtml10" alt="Valid HTML5" title="{$this->name}{$this->strPro} Admin Page is certified Valid HTML5" height="31" onmouseover="Tip('{$this->name}{$this->strPro} Admin Page is certified Valid HTML5, with no errors in the HTML code generated by the plugin.')" onmouseout="UnTip()" width="88" class="alignright"/></a>
</h2>
EOF1;
        $permaStructure = get_option('permalink_structure');
        if (empty($permaStructure)) {
          $permalink = admin_url('options-permalink.php');
          ?>
          <div class='error' style='padding:10px;margin:10px;color:#a00;font-weight:500;background-color:#fee;' id="permalinks">
            <p><strong>Permalinks</strong> are not enabled on your blog, which this plugin needs. Please <a href='<?php echo $permalink; ?>'>enable a permalink structure</a> for your blog from <strong><a href='<?php echo $permalink; ?>'>Settings &rarr; Permalinks</a></strong>.<br> Any structure (other than the ugly default structure using <code><?php echo site_url(); ?>/?p=123</code>) will do.</p>
          </div>
          <?php
        }
        ?>
        <h3>
          <?php
          _e('Introduction', 'easy-pagespeed');
          ?>
        </h3>
        <div style="width:45%;float:left;display:inline-block;padding:10px">
          <p><strong>Easy PageSpeed</strong> does just two of the things that most caching plugins do:</p>
          <ol>
            <li>
              Rewrite this links to your static resources so that they are server from your Content Delivery Network (CDN).
            </li>
            <li>
              Remove query strings (version number) from static resources so that they can be locally cached at client browsers.
            </li>
          </ol>
          <p>In the Pro version of this plugin, you can specify multiple CDNs (or groups of CDNs) and have specific types of files served from a specifc CDN (or group). <a href='#' class="pro-version">More information.</a></p>
        </div>
        <div style="width:45%;float:left;display:inline-block;padding:10px">
          <table class="form-table">
            <tr style="vertical-align:middle">
              <?php
              $ez->renderHeadText($this->isPro);
              ?>
            </tr>
          </table>
        </div>
        <div style='clear:both'></div>
        <?php
        $this->printInfoDivs();
        ?>
        <form method='post' action='#'>
          <?php
          $this->renderNonce();
          ?>
          <h3>
            <?php
            printf(__('Settings', 'easy-pagespeed'));
            ?>
          </h3>
          <div style="width:50%;float:left;display:inline-block;padding:10px">
            <?php
            $firstCol = array('pause_cdn',
                'kill_qs',
                'show_timing',
                'enable_cdn',
                'cdn_server');
            foreach ($firstCol as $k) {
              if (!empty($this->ezOptions[$k])) {
                $this->ezOptions[$k]->render();
              }
            }
            ?>
          </div>
          <div style="width:45%;float:right;display:inline-block;">
            <?php
            $this->printInfoLinks();
            ?>
            <br>
          </div>
          <div style="clear:both"></div>
          <div style='padding-bottom:20px;text-align:center'>
            <?php
            $this->ezOptions['kill_author']->render();
            ?>
          </div>
          <?php
          $this->printProSection()
          ?>
          <div class="submit">
            <?php
            $this->renderSubmitButtons();
            ?>
          </div>
        </form>

        <?php
        $ez->renderWhyPro();
        $ez->renderSupport();
        $ez->renderTailText();
        ?>
      </div>
      <?php
    }

    /**
     * Picks a random CDN from the CDN list (if available) if the given URL
     * can be served from one.<p>
     * Called from the call-back of a preg_replace_callback (addCacheType)
     *
     * @param string $request The request part of a URL match
     * @return string  CDN or empty
     */
    function getCDN($request) {
      if (empty($this->options['enable_cdn'])) {
        return;
      }
      if (empty($this->options['cdn_server'])) {
        return;
      }
      $ext = pathinfo($request, PATHINFO_EXTENSION);
      if (!in_array($ext, self::$cdnExt)) {
        return;
      }
      return self::$cdnServer;
    }

    function killQS($url) {
      $url[2] = preg_replace(self::$qsRegEx, '\\1\\2', $url[2]);
      $urlEx = $url[1] . $url[2];
      return $urlEx;
    }

    function addCdn($url) {
      $urlEx = $url[1] . $url[2];
      $request = "/" . rtrim($url[2], "'\"");
      $cdn = $this->getCDN($request);
      if (!empty($cdn)) {
        $urlEx = $url[1][0] . $cdn . $url[2];
      }
      return $urlEx;
    }

    function killQSandAddCdn($url) {
      $url[2] = preg_replace(self::$qsRegEx, '\\1\\2', $url[2]);
      $urlEx = $url[1] . $url[2];
      $request = "/" . rtrim($url[2], "'\"");
      $cdn = $this->getCDN($request);
      if (!empty($cdn)) {
        $urlEx = $url[1][0] . $cdn . $url[2];
      }
      return $urlEx;
    }

    /**
     * Buffer the output to be passed to the endBuffer function.
     */
    function startBuffer() {
      ob_start(array($this, 'endBuffer'));
    }

    /**
     * Removes the query string.<p>
     * Adds CDN<p>.
     * Optionally times execution.
     * @param string $html HTML string to be cached.
     * @return string Minified, cacheable HTML string
     */
    function endBuffer($html) {
      if (empty($html)) {
        return $html;
      }
      $regex = "#(['\"]" . self::$siteUrl . ")(.*?['\"])#";
      $t0 = microtime(true);
      if (!empty($this->options['kill_qs']) && !empty($this->options['enable_cdn'])) {
        $html = preg_replace_callback($regex, array($this, 'killQSandAddCdn'), $html);
      }
      else if (!empty($this->options['kill_qs'])) {
        $html = preg_replace_callback($regex, array($this, 'killQS'), $html);
      }
      else if (!empty($this->options['enable_cdn'])) {
        $html = preg_replace_callback($regex, array($this, 'addCdn'), $html);
      }
      if (!empty($this->options['show_timing'])) {
        $t1 = microtime(true);
        $t = $t1 - $t0;
        return $html . "<!-- Time taken in Easy PageSpeed endBuffer function = $t seconds. -->";
      }
      else {
        return $html;
      }
    }

    // WordPress Actions and Filters

    /**
     * Action call-back to parse_request.<p>
     * Cache a URL, serve and die if possible.
     * If not, return empty
     *
     * @return Bool Empty if REQUEST_URI is not a cached or cacheable.
     */
    function parseRequest() {
      $this->startBuffer();
    }

    /**
     * Call back to print admin footer scripts
     */
    function adminPrintFooterScripts() {
      parent::adminPrintFooterScripts();
      ?>
      <script>
        jQuery(document).ready(function () {
          if (!Array.prototype.last) {
            Array.prototype.last = function () {
              return this[this.length - 1];
            };
          }
          var divClasses = '.cdn, .what,.why, .pro-version';
          jQuery('body').on('click', divClasses, function () {
            var id = '#' + this.className.split(" ").last();
            jQuery(".info-div").not(id).hide('show');
            jQuery(id).toggle('show');
          });
        });
      </script>
      <?php
    }

  }

}
